@@ -19,6 +19,7 @@ BEGIN {
 }
 
 our @EXPORT_OK = qw<is_set
+  get_uint2
   get_uint6
   get_uint12
   get_uint16
@@ -34,6 +35,15 @@ sub is_set {
     return substr( $data, $offset, 1 ) == 1;
 }
 
+sub get_uint2 {
+    my ( $data, $offset ) = @_;
+
+    return unpack(
+        "C",
+        _get_bits_with_padding( $data, 8, $offset, 2 )
+    );
+}
+
 sub get_uint6 {
     my ( $data, $offset ) = @_;
 
